/*
 * This program will list the qdiscs and classes plus some statistics information for each.
 */
#include <unistd.h>
#include <stdlib.h>

#include <glib.h>

#include "lql-con.h"
#include "lql-interface.h"
#include "lql-qdisc.h"
#include "lql-class.h"

int
main(int argc,
     char *argv[])
{
	LQLCon *con = NULL;
	LQLInterface *interface = NULL;
	GList *list;

	g_type_init();

	con = lql_con_new();

	// Get the interfaces and find the index of eth0.
	list = lql_con_get_interfaces(con);
	while (list != NULL) {
		interface = list->data;
		if (g_strcasecmp("eth0", lql_interface_get_name(interface)) == 0) {
			break;
		} else {
			g_object_unref(interface);
		}

		list = list->next;
	}
	g_list_free(list);

	// Get the number of QDiscs.
	g_print("-------------------------------------\n");
	list = lql_con_get_qdiscs(con, lql_interface_get_ifIndex(interface));
	g_print("Number of QDiscs: %i\n", g_list_length(list));
	while (list != NULL) {
		LQLQDisc *qdisc = NULL;
		qdisc = (LQLQDisc *)list->data;
		if (qdisc != NULL) {
			lql_qdisc_print(qdisc);
			lql_qdisc_updateStats(qdisc, con);
			lql_qdisc_printStats(qdisc);
		
			g_object_unref(qdisc);
		} else {
			g_print("NULL QDisc\n");
		}

		list = list->next;
	}
	g_list_free(list);

	// Get the number of Classes.
	g_print("-------------------------------------\n");
	list = lql_con_get_classes(con, lql_interface_get_ifIndex(interface));
	g_print("Number of classes: %i\n", g_list_length(list));

	while (list != NULL) {
		LQLClass *class = NULL;
		class = (LQLClass *)list->data;
		if (class != NULL) {
			lql_class_print(class);
			lql_class_updateStats(class, con);
			lql_class_printStats(class);

			g_object_unref(class);
		} else {
			g_print("NULL class.\n");
		}
		
		list = list->next;
	}
	g_list_free(list);

	// Cleanup.
	g_object_unref(interface);
	g_object_unref(con);

	return(0);
}
