/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "lql-class-priomap.h"
#include "lql-class-priomap-private.h"
#include "lql_class_priomap_helper.h"

/********************************************************************
 * Non-Virutal API
 ********************************************************************/

/**
 * lql_class_priomap_new:
 *
 * Build a new instance of LQLClassPriomap. 
 * 
 * Return value: A new LQLClassPriomap instance.
 **/
LQLClassPriomap*
lql_class_priomap_new(void)
{
        return(LQL_CLASS_PRIOMAP(g_object_new(LQL_TYPE_CLASS_PRIOMAP, NULL)));
}

/********************************************************************
 * Virtual API
 ********************************************************************/

/**
 * lql_class_priomap_fill_from_nlhdr:
 * @self: The LQLClass object to work with.
 * @hdr: The nl packet to get the queue information from.
 *
 * This method takes a nlmsg from the kernel and set this
 * objects instance variables to match it. This is used
 * to create objects that match the ones currently active
 * in the system. This overrides the default LQLClass
 * implementation of this method.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_class_priomap_fill_from_nlhdr(LQLClass *self,
				  const struct nlmsghdr *hdr)
{
	g_assert(self != NULL);
	g_assert(hdr != NULL);

	return(lql_class_priomap_helper_fill_from_nlhdr(LQL_CLASS_PRIOMAP(self), hdr));
}

/********************************************************************
 * Object stuff.
 ********************************************************************/
static void
lql_class_priomap_instance_init(GTypeInstance *instance,
			    gpointer g_class)
{
	LQLClassPriomap *self = (LQLClassPriomap *)instance;
	self->private = g_new0(LQLClassPriomapPrivate, 1);
	self->private->dispose_has_run = FALSE;
}

static void
lql_class_priomap_set_property(GObject *object,
			   guint property_id,
			   const GValue *value,
			   GParamSpec *pspec)
{
//        LQLClassPriomap *self = (LQLClassPriomap *) object;

        switch (property_id) {
        default:
                g_assert(FALSE);
                break;
        }
}

static void
lql_class_priomap_get_property(GObject *object,
                           guint property_id,
                           GValue *value,
                           GParamSpec *pspec)
{
//        LQLClassPriomap *self = (LQLClassPriomap *) object;

        switch (property_id) {
        default:
                g_assert(FALSE);
                break;
        }
}

static GObject *
lql_class_priomap_constructor(GType type,
                          guint n_construct_properties,
                          GObjectConstructParam *construct_properties)
{
        GObject *obj = NULL;

        {
                /* Invoke parent constructor. */
		LQLClassPriomapClass *klass;
		GObjectClass *parent_class;  
		klass = LQL_CLASS_PRIOMAP_CLASS(g_type_class_peek(LQL_TYPE_CLASS_PRIOMAP));
		parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));
		obj = parent_class->constructor(type,
						n_construct_properties,
						construct_properties);
        }
        
        /* do stuff. */

        return obj;
}

static void
lql_class_priomap_dispose(GObject *obj)
{
        LQLClassPriomap *self = (LQLClassPriomap *)obj;

        if (self->private->dispose_has_run) {
                /* If dispose did already run, return. */
                return;
        }
        /* Make sure dispose does not run twice. */
        self->private->dispose_has_run = TRUE;

        /* 
         * In dispose, you are supposed to free all types referenced from this
         * object which might themselves hold a reference to self. Generally,
         * the most simple solution is to unref all members on which you own a 
         * reference.
         */
}

static void
lql_class_priomap_finalize(GObject *obj)
{
        LQLClassPriomap *self = (LQLClassPriomap *)obj;

        /*
         * Here, complete object destruction.
         */

        
        g_free (self->private);
}


static void
lql_class_priomap_class_init(gpointer g_class,
			 gpointer g_class_data)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
        LQLClassClass *klass = LQL_CLASS_CLASS(g_class);

        gobject_class->set_property = lql_class_priomap_set_property;
        gobject_class->get_property = lql_class_priomap_get_property;
        gobject_class->dispose = lql_class_priomap_dispose;
        gobject_class->finalize = lql_class_priomap_finalize;
        gobject_class->constructor = lql_class_priomap_constructor;

	/* Set the virtual methods. */
	klass->fill_from_nlhdr = lql_class_priomap_fill_from_nlhdr;
	klass->print = lql_class_priomap_helper_print;

	/* Setup the properties. */
}

GType lql_class_priomap_get_type (void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (LQLClassPriomapClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        lql_class_priomap_class_init,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (LQLClassPriomap),
                        0,      /* n_preallocs */
                        lql_class_priomap_instance_init    /* instance_init */
                };
                type = g_type_register_static (LQL_TYPE_CLASS,
                                               "LQLClassPriomap",
                                               &info, 0);
        }
        return type;
}
