/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "lql-classifier-u32.h"
#include "lql-classifier-u32-private.h"
#include "lql_classifier_u32_helper.h"

/********************************************************************
 * Non-Virutal API
 ********************************************************************/

/**
 * lql_classifier_u32_new:
 *
 * Build a new instance of LQLClassifierU32. 
 * 
 * Return value: A new LQLClassifierU32 instance.
 **/
LQLClassifierU32*
lql_classifier_u32_new(void)
{
        return(LQL_CLASSIFIER_U32(g_object_new(LQL_TYPE_CLASSIFIER_U32, NULL)));
}

/**
 * lql_classifier_u32_get_hash:
 * @self: The LQLClassifierU32 object to work with.
 * 
 * Get the hash.
 * 
 * Return value: The hash for this filter.
 **/
guint32
lql_classifier_u32_get_hash(LQLClassifierU32 *self)
{
        guint32 hash = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(0);
        }
        
        g_object_get(LQL_CLASSIFIER(self), "hash", &hash, NULL);

        return(hash);
}

/**
 * lql_classifier_u32_set_hash:
 * @self: The LQLClassifier object to work with.
 * @hash: The hash value.
 * 
 * Set the hash value for this classifier.
 * 
 * Return value: TRUE on success, FALSE on error.
 **/
gboolean
lql_classifier_u32_set_hash(LQLClassifierU32 *self,
			    guint32 hash)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASSIFIER(self), "hash", hash, NULL);
        
        return(TRUE);
}

guint32
lql_classifier_u32_get_link(LQLClassifierU32 *self)
{
        guint32 link = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(0);
        }
        
        g_object_get(LQL_CLASSIFIER(self), "link", &link, NULL);

        return(link);
}

gboolean
lql_classifier_u32_set_link(LQLClassifierU32 *self,
			    guint32 link)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASSIFIER(self), "link", link, NULL);
        
        return(TRUE);
}

guint32
lql_classifier_u32_get_divisor(LQLClassifierU32 *self)
{
        guint32 divisor = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(0);
        }
        
        g_object_get(LQL_CLASSIFIER(self), "divisor", &divisor, NULL);

        return(divisor);
}

gboolean
lql_classifier_u32_set_divisor(LQLClassifierU32 *self,
			       guint32 divisor)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASSIFIER(self), "divisor", divisor, NULL);
        
        return(TRUE);
}

guint32
lql_classifier_u32_get_hashMask(LQLClassifierU32 *self)
{
        guint32 hashMask = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(0);
        }
        
        g_object_get(LQL_CLASSIFIER(self), "hash-mask", &hashMask, NULL);

        return(hashMask);
}

gboolean
lql_classifier_u32_set_hashMask(LQLClassifierU32 *self,
				guint32 hashMask)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASSIFIER(self), "hash-mask", hashMask, NULL);
        
        return(TRUE);
}

guint32
lql_classifier_u32_get_hashOffset(LQLClassifierU32 *self)
{
        guint32 hashOffset = 0;

        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(0);
        }
        
        g_object_get(LQL_CLASSIFIER(self), "hash-offset", &hashOffset, NULL);

        return(hashOffset);
}

gboolean
lql_classifier_u32_set_hashOffset(LQLClassifierU32 *self,
				  guint32 hashOffset)
{
        g_assert(self != NULL);

        if (self->private->dispose_has_run) {
                return(FALSE);
        }
        
        g_object_set(LQL_CLASSIFIER(self), "hash-offset", hashOffset, NULL);
        
        return(TRUE);
}

guint16
lql_classifier_u32_get_handle(LQLClassifierU32 *self)
{
	g_assert(self != NULL);

	return(lql_classifier_u32_helper_get_handle(self));
}

gboolean
lql_classifier_u32_set_handle(LQLClassifierU32 *self,
			      guint16 handle)
{
	g_assert(self != NULL);

	return(lql_classifier_u32_helper_set_handle(self, handle));
}

guint16
lql_classifier_u32_get_hashHandle(LQLClassifierU32 *self)
{
	g_assert(self != NULL);

	return(lql_classifier_u32_helper_get_hashHandle(self));
}

gboolean
lql_classifier_u32_set_hashHandle(LQLClassifierU32 *self,
				  guint16 hashHandle)
{
	g_assert(self != NULL);

	return(lql_classifier_u32_helper_set_hashHandle(self, hashHandle));
}

guint16
lql_classifier_u32_get_hashNode(LQLClassifierU32 *self)
{
	g_assert(self != NULL);

	return(lql_classifier_u32_helper_get_hashNode(self));
}

gboolean
lql_classifier_u32_set_hashNode(LQLClassifierU32 *self,
				guint16 hashNode)
{
	g_assert(self != NULL);

	return(lql_classifier_u32_helper_set_hashNode(self, hashNode));
}

gboolean
lql_classifier_u32_add_match(LQLClassifierU32 *self,
			     LQLU32Match *sel)
{
	g_assert(self != NULL);
	g_assert(sel != NULL);

	return(lql_classifier_u32_helper_add_match(self, sel));
}

gboolean
lql_classifier_u32_add_match_ip_dst(LQLClassifierU32 *self,
				    const gchar *ip)
{
	g_assert(self != NULL);
	g_assert(ip != NULL);

	return(lql_classifier_u32_helper_add_match_ip_dst(self, ip));
}

LQLU32Match*
lql_classifier_u32_get_match(LQLClassifierU32 *self,
			     int position)
{
	g_assert(self != NULL);
	g_assert(position >= 0);

	return(lql_classifier_u32_helper_get_match(self, position));
}

int
lql_classifier_u32_get_match_len(LQLClassifierU32 *self)
{
	g_assert(self != NULL);

	return(lql_classifier_u32_helper_get_match_len(self));
}

/********************************************************************
 * Virtual API
 ********************************************************************/

/**
 * lql_classifier_u32_fill_from_nlhdr:
 * @self: The LQLClassifier object to work with.
 * @hdr: The nl packet to get the queue information from.
 *
 * This method takes a nlmsg from the kernel and set this
 * objects instance variables to match it. This is used
 * to create objects that match the ones currently active
 * in the system. This overrides the default LQLClassifier
 * implementation of this method.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_classifier_u32_fill_from_nlhdr(LQLClassifier *self,
				      const struct nlmsghdr *hdr)
{
	g_assert(self != NULL);
	g_assert(hdr != NULL);

	return(lql_classifier_u32_helper_fill_from_nlhdr(LQL_CLASSIFIER_U32(self), hdr));
}

/**
 * lql_classifier_u32_add:
 * @self: The LQLClassifier object to add to the system.
 * @con: The LQLCon connection to use when adding this queueing discipline.
 *
 * Add this HTB queueing discipline to the system. This overrides
 * the default implementation of this function in LQLClassifier.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_classifier_u32_add(LQLClassifier *self,
			  LQLCon *con)
{
	g_assert(self != NULL);
	g_assert(con != NULL);

	return(lql_classifier_u32_helper_add(LQL_CLASSIFIER_U32(self), con));
}

/**
 * lql_classifier_u32_modify:
 * @self: The LQLClassifier object to modify.
 * @con: The LQLCon connection to use when adding this classifier.
 *
 * Modify the HTB queueing discipline in the system to match the current
 * object state. This method overrides the default
 * implementation in LQLClassifier.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_classifier_u32_modify(LQLClassifier *self,
			     LQLCon *con)
{
	g_assert(self != NULL);
	g_assert(con != NULL);

	return(lql_classifier_u32_helper_modify(LQL_CLASSIFIER_U32(self), con));
}

/********************************************************************
 * Object stuff.
 ********************************************************************/
enum {
	LQL_CLASSIFIER_U32_HASH = 1,
	LQL_CLASSIFIER_U32_LINK = 2,
	LQL_CLASSIFIER_U32_DIVISOR = 3,
	LQL_CLASSIFIER_U32_HASHMASK = 4,
	LQL_CLASSIFIER_U32_HASHOFFSET = 5,
};

static void
lql_classifier_u32_instance_init(GTypeInstance *instance,
				 gpointer g_class)
{
	LQLClassifierU32 *self = (LQLClassifierU32 *)instance;
	self->private = g_new0(LQLClassifierU32Private, 1);
	self->private->dispose_has_run = FALSE;
}

static void
lql_classifier_u32_set_property(GObject *object,
				   guint property_id,
				   const GValue *value,
				   GParamSpec *pspec)
{
        LQLClassifierU32 *self = (LQLClassifierU32 *) object;

        switch (property_id) {
	case LQL_CLASSIFIER_U32_HASH:
	{
		self->private->hash = g_value_get_uint(value);
		break;
	}
	case LQL_CLASSIFIER_U32_LINK:
	{
		self->private->link = g_value_get_uint(value);
		break;
	}
	case LQL_CLASSIFIER_U32_DIVISOR:
	{
		self->private->divisor = g_value_get_uint(value);
		break;
	}
	case LQL_CLASSIFIER_U32_HASHMASK:
	{
		self->private->hashMask = g_value_get_uint(value);
		break;
	}
	case LQL_CLASSIFIER_U32_HASHOFFSET:
	{
		self->private->hashOffset = g_value_get_uint(value);
		break;
	}
        default:
                g_assert(FALSE);
                break;
        }
}

static void
lql_classifier_u32_get_property(GObject *object,
				   guint property_id,
				   GValue *value,
				   GParamSpec *pspec)
{
        LQLClassifierU32 *self = (LQLClassifierU32 *) object;

        switch (property_id) {
	case LQL_CLASSIFIER_U32_HASH:
	{
		g_value_set_uint(value, self->private->hash);
		break;
	}
	case LQL_CLASSIFIER_U32_LINK:
	{
		g_value_set_uint(value, self->private->link);
		break;
	}
	case LQL_CLASSIFIER_U32_DIVISOR:
	{
		g_value_set_uint(value, self->private->divisor);
		break;
	}
	case LQL_CLASSIFIER_U32_HASHMASK:
	{
		g_value_set_uint(value, self->private->hashMask);
		break;
	}
	case LQL_CLASSIFIER_U32_HASHOFFSET:
	{
		g_value_set_uint(value, self->private->hashOffset);
		break;
	}
        default:
                g_assert(FALSE);
                break;
        }
}

static GObject *
lql_classifier_u32_constructor(GType type,
				  guint n_construct_properties,
				  GObjectConstructParam *construct_properties)
{
        GObject *obj = NULL;

        {
                /* Invoke parent constructor. */
		LQLClassifierU32Class *klass;
		GObjectClass *parent_class;  
		klass = LQL_CLASSIFIER_U32_CLASS(g_type_class_peek(LQL_TYPE_CLASSIFIER_U32));
		parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));
		obj = parent_class->constructor(type,
						n_construct_properties,
						construct_properties);
        }
        
        /* do stuff. */

        return obj;
}

static void
lql_classifier_u32_dispose(GObject *obj)
{
        LQLClassifierU32 *self = (LQLClassifierU32 *)obj;

        if (self->private->dispose_has_run) {
                /* If dispose did already run, return. */
                return;
        }
        /* Make sure dispose does not run twice. */
        self->private->dispose_has_run = TRUE;

        /* 
         * In dispose, you are supposed to free all types referenced from this
         * object which might themselves hold a reference to self. Generally,
         * the most simple solution is to unref all members on which you own a 
         * reference.
         */
}

static void
lql_classifier_u32_finalize(GObject *obj)
{
        LQLClassifierU32 *self = (LQLClassifierU32 *)obj;

        /*
         * Here, complete object destruction.
         */
        
        g_free (self->private);
}


static void
lql_classifier_u32_class_init(gpointer g_class,
				 gpointer g_class_data)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
        LQLClassifierClass *klass = LQL_CLASSIFIER_CLASS(g_class);
        GParamSpec *pspec = NULL;

        gobject_class->set_property = lql_classifier_u32_set_property;
        gobject_class->get_property = lql_classifier_u32_get_property;
        gobject_class->dispose = lql_classifier_u32_dispose;
        gobject_class->finalize = lql_classifier_u32_finalize;
        gobject_class->constructor = lql_classifier_u32_constructor;

	/* Set the virtual methods. */
	klass->fill_from_nlhdr = lql_classifier_u32_fill_from_nlhdr;
	klass->add = lql_classifier_u32_add;
	klass->modify = lql_classifier_u32_modify;

	/* Setup the properties. */
        pspec = g_param_spec_uint("hash",
				  "hash",
				  "Set/Get the hash",
				  0 /* minimum value */,
				  G_MAXUINT32 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASSIFIER_U32_HASH,
                                        pspec);

        pspec = g_param_spec_uint("link",
				  "link",
				  "Set/Get the link",
				  0 /* minimum value */,
				  G_MAXUINT32 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASSIFIER_U32_LINK,
                                        pspec);

        pspec = g_param_spec_uint("divisor",
				  "divisor",
				  "Set/Get the divisor",
				  0 /* minimum value */,
				  G_MAXUINT32 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASSIFIER_U32_DIVISOR,
                                        pspec);

        pspec = g_param_spec_uint("hash-mask",
				  "hash-mask",
				  "Set/Get the hashMask",
				  0 /* minimum value */,
				  G_MAXUINT32 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASSIFIER_U32_HASHMASK,
                                        pspec);

        pspec = g_param_spec_uint("hash-offset",
				  "hash-offset",
				  "Set/Get the hashOffset",
				  0 /* minimum value */,
				  G_MAXUINT32 /* maximum value */,
				  0 /* default value */,
				  G_PARAM_READWRITE);
        g_object_class_install_property(gobject_class,
                                        LQL_CLASSIFIER_U32_HASHOFFSET,
                                        pspec);
}

GType
lql_classifier_u32_get_type(void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (LQLClassifierU32Class),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        lql_classifier_u32_class_init,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (LQLClassifierU32),
                        0,      /* n_preallocs */
                        lql_classifier_u32_instance_init    /* instance_init */
                };
                type = g_type_register_static (LQL_TYPE_CLASSIFIER,
                                               "LQLClassifierU32",
                                               &info, 0);
        }
        return type;
}
