/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include "lql-qdisc-pfifofast.h"
#include "lql-qdisc-pfifofast-private.h"
#include "lql_qdisc_pfifofast_helper.h"

/********************************************************************
 * Non-Virutal API
 ********************************************************************/

/**
 * lql_qdisc_pfifofast_new:
 *
 * Build a new instance of LQLQDiscPFIFOFast. 
 * 
 * Return value: A new LQLQDiscPFIFOFast instance.
 **/
LQLQDiscPFIFOFast*
lql_qdisc_pfifofast_new(void)
{
        return(LQL_QDISC_PFIFOFAST(g_object_new(LQL_TYPE_QDISC_PFIFOFAST, NULL)));
}

/********************************************************************
 * Virtual API
 ********************************************************************/

/**
 * lql_qdisc_pfifofast_fill_from_nlhdr:
 * @self: The LQLQDisc object to work with.
 * @hdr: The nl packet to get the queue information from.
 *
 * This method takes a nlmsg from the kernel and set this
 * objects instance variables to match it. This is used
 * to create objects that match the ones currently active
 * in the system. This overrides the default LQLQDisc
 * implementation of this method.
 *
 * Returns: TRUE on success, FALSE on failure.
 **/
gboolean
lql_qdisc_pfifofast_fill_from_nlhdr(LQLQDisc *self,
			      const struct nlmsghdr *hdr)
{
	g_assert(self != NULL);
	g_assert(hdr != NULL);

	return(lql_qdisc_pfifofast_helper_fill_from_nlhdr(LQL_QDISC_PFIFOFAST(self), hdr));
}

/********************************************************************
 * Object stuff.
 ********************************************************************/
static void
lql_qdisc_pfifofast_instance_init(GTypeInstance *instance,
			    gpointer g_class)
{
	LQLQDiscPFIFOFast *self = (LQLQDiscPFIFOFast *)instance;
	self->private = g_new0(LQLQDiscPFIFOFastPrivate, 1);
	self->private->dispose_has_run = FALSE;
}

static void
lql_qdisc_pfifofast_set_property(GObject *object,
			   guint property_id,
			   const GValue *value,
			   GParamSpec *pspec)
{
        //LQLQDiscPFIFOFast *self = (LQLQDiscPFIFOFast *) object;

        switch (property_id) {
        default:
                g_assert(FALSE);
                break;
        }
}

static void
lql_qdisc_pfifofast_get_property(GObject *object,
                           guint property_id,
                           GValue *value,
                           GParamSpec *pspec)
{
        //LQLQDiscPFIFOFast *self = (LQLQDiscPFIFOFast *) object;

        switch (property_id) {
        default:
                g_assert(FALSE);
                break;
        }
}

static GObject *
lql_qdisc_pfifofast_constructor(GType type,
                          guint n_construct_properties,
                          GObjectConstructParam *construct_properties)
{
        GObject *obj = NULL;

        {
                /* Invoke parent constructor. */
		LQLQDiscPFIFOFastClass *klass;
		GObjectClass *parent_class;  
		klass = LQL_QDISC_PFIFOFAST_CLASS(g_type_class_peek(LQL_TYPE_QDISC_PFIFOFAST));
		parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));
		obj = parent_class->constructor(type,
						n_construct_properties,
						construct_properties);
        }
        
        /* do stuff. */

        return obj;
}

static void
lql_qdisc_pfifofast_dispose(GObject *obj)
{
        LQLQDiscPFIFOFast *self = (LQLQDiscPFIFOFast *)obj;

        if (self->private->dispose_has_run) {
                /* If dispose did already run, return. */
                return;
        }
        /* Make sure dispose does not run twice. */
        self->private->dispose_has_run = TRUE;

        /* 
         * In dispose, you are supposed to free all types referenced from this
         * object which might themselves hold a reference to self. Generally,
         * the most simple solution is to unref all members on which you own a 
         * reference.
         */
}

static void
lql_qdisc_pfifofast_finalize(GObject *obj)
{
        LQLQDiscPFIFOFast *self = (LQLQDiscPFIFOFast *)obj;

        /*
         * Here, complete object destruction.
         */
        
        g_free (self->private);
}


static void
lql_qdisc_pfifofast_class_init(gpointer g_class,
			 gpointer g_class_data)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
        LQLQDiscClass *klass = LQL_QDISC_CLASS(g_class);

        gobject_class->set_property = lql_qdisc_pfifofast_set_property;
        gobject_class->get_property = lql_qdisc_pfifofast_get_property;
        gobject_class->dispose = lql_qdisc_pfifofast_dispose;
        gobject_class->finalize = lql_qdisc_pfifofast_finalize;
        gobject_class->constructor = lql_qdisc_pfifofast_constructor;

	/* Set the virtual methods. */
	klass->fill_from_nlhdr = lql_qdisc_pfifofast_fill_from_nlhdr;
	klass->print = lql_qdisc_pfifofast_helper_print;

	/* Setup the properties. */
}

GType lql_qdisc_pfifofast_get_type (void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (LQLQDiscPFIFOFastClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        lql_qdisc_pfifofast_class_init,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (LQLQDiscPFIFOFast),
                        0,      /* n_preallocs */
                        lql_qdisc_pfifofast_instance_init    /* instance_init */
                };
                type = g_type_register_static(LQL_TYPE_QDISC,
					      "LQLQDiscPFIFOFast",
					      &info, 0);
        }
        return type;
}
