/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#ifndef _LQL_QDISC_H_
#define _LQL_QDISC_H_

#include <glib-object.h>
#include "nl.h"
#include "lql-con.h"
#include "lql-element.h"

#define LQL_TYPE_QDISC (lql_qdisc_get_type ())
#define LQL_QDISC(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), LQL_TYPE_QDISC, LQLQDisc))
#define LQL_QDISC_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), LQL_TYPE_QDISC, LQLQDiscClass))
#define LQL_IS_QDISC(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), LQL_TYPE_QDISC))
#define LQL_IS_QDISC_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), LQL_TYPE_QDISC))
#define LQL_QDISC_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), LQL_TYPE_QDISC, LQLQDiscClass))

typedef struct _LQLQDisc LQLQDisc;
typedef struct _LQLQDiscClass LQLQDiscClass;
typedef struct _LQLQDiscPrivate LQLQDiscPrivate;

struct _LQLQDisc {
	LQLElement parent;

	/* Instance members */
        LQLQDiscPrivate *private;
};

struct _LQLQDiscClass {
	LQLElementClass parent;

	/* Class members */
	gboolean (*fill_from_nlhdr) (LQLQDisc *qdisc, const struct nlmsghdr *hdr);
	gboolean (*add) (LQLQDisc *qdisc, LQLCon *con);
	gboolean (*modify) (LQLQDisc *qdisc, LQLCon *con);
	gboolean (*delete) (LQLQDisc *qdisc, LQLCon *con);
	void (*print) (LQLQDisc *qdisc);
	void (*printStats) (LQLQDisc *qdisc);
	gboolean (*updateStats) (LQLQDisc *qdisc, LQLCon *con);
};

/* used by LQL_TYPE_QDISC */
GType lql_qdisc_get_type (void);

/* Non-virtual API. */
LQLQDisc* lql_qdisc_new(void);

gboolean lql_qdisc_set_parent(LQLQDisc *self, LQLElement *parent);
gboolean lql_qdisc_set_parent_class_id(LQLQDisc *self, guint16 class);
gboolean lql_qdisc_set_id(LQLQDisc *self, guint16 id);
guint16 lql_qdisc_get_id(LQLQDisc *self);

guint64 lql_qdisc_get_bytes(LQLQDisc *self);
guint32 lql_qdisc_get_packets(LQLQDisc *self);
guint32 lql_qdisc_get_drops(LQLQDisc *self);
guint32 lql_qdisc_get_overlimits(LQLQDisc *self);
guint32 lql_qdisc_get_bps(LQLQDisc *self);
guint32 lql_qdisc_get_pps(LQLQDisc *self);
guint32 lql_qdisc_get_qlen(LQLQDisc *self);
guint32 lql_qdisc_get_backlog(LQLQDisc *self);

/* Virtual API. */
gboolean lql_qdisc_fill_from_nlhdr(LQLQDisc *self, const struct nlmsghdr *hdr);
gboolean lql_qdisc_add(LQLQDisc *self, LQLCon *con);
gboolean lql_qdisc_modify(LQLQDisc *self, LQLCon *con);
gboolean lql_qdisc_delete(LQLQDisc *self, LQLCon *con);
void lql_qdisc_print(LQLQDisc *self);
void lql_qdisc_printStats(LQLQDisc *self);
gboolean lql_qdisc_updateStats(LQLQDisc *self, LQLCon *con);

#endif /* _LQL_QDISC_H_ */
