/*
 * Copyright (C) 2004 Dan Siemon <dan@coverfire.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <unistd.h>

#include <glib.h>

#include "nl.h"
#include "lql-con.h"
#include "lql-class.h"
#include "lql-class-private.h"

#define BUFF 4096

gboolean
lql_class_helper_delete(LQLClass *class,
			LQLCon *con)
{
	g_assert(class != NULL);
	g_assert(con != NULL);
	
	TcPacket *pkt = NULL;
	struct nlmsghdr *hdr = NULL;

	g_assert(class != NULL);
	g_assert(con != NULL);

	pkt = nl_tcpacket_new();
	hdr = nl_tcpacket_get_nlmsg(pkt);

	nl_nlmsg_set_type(hdr, RTM_DELTCLASS);
	nl_nlmsg_set_flags(hdr, NLM_F_REQUEST | NLM_F_ACK);
	nl_nlmsg_set_pid(hdr, getpid());
	nl_nlmsg_set_seq(hdr, lql_con_get_inc_sequence(LQL_CON(con)));

	nl_tcmsg_set_family(hdr, AF_UNSPEC);
	nl_tcmsg_set_ifindex(hdr, lql_element_get_ifIndex(LQL_ELEMENT(class)));
	nl_tcmsg_set_parent(hdr, lql_element_get_parentHandle(LQL_ELEMENT(class)));
	nl_tcmsg_set_handle(hdr, lql_element_get_handle(LQL_ELEMENT(class)));

	if (!nl_tcpacket_do_command(pkt, lql_con_get_nlcon(LQL_CON(con)))) {
		return(FALSE);
	}

	g_free(pkt);

	return(TRUE);
}

void
lql_class_helper_print(LQLClass *self)
{
	g_print("------------ Class ------------\n");
	g_print("Handle: %u\n", lql_element_get_handle(LQL_ELEMENT(self)));
	g_print("Parent: %i\n", lql_element_get_parentHandle(LQL_ELEMENT(self)));
	g_print("ID: %i\n", lql_class_get_id(self));
	g_print("Interface index: %i\n", lql_element_get_ifIndex(LQL_ELEMENT(self)));
	g_print("Priority: %i\n", lql_class_get_priority(self));	
}

void
lql_class_helper_printStats(LQLClass *self)
{
	g_assert(self != NULL);
	g_assert(LQL_IS_CLASS(self));

        if (self->private->dispose_has_run) {
                return;
        }

	g_print("Packets: %u\n",
		lql_class_get_packets(LQL_CLASS(self)));
	g_print("Bytes: %lld\n",
		lql_class_get_bytes(LQL_CLASS(self)));
	g_print("Drops: %u\n",
		lql_class_get_drops(LQL_CLASS(self)));
	g_print("Overlimits: %u\n",
		lql_class_get_overlimits(LQL_CLASS(self)));
	g_print("BPS: %u\n",
		lql_class_get_bps(LQL_CLASS(self)));
	g_print("PPS: %u\n",
		lql_class_get_pps(LQL_CLASS(self)));
	g_print("Queue Length: %u\n",
		lql_class_get_qlen(LQL_CLASS(self)));
	g_print("Backlog: %u\n",
		lql_class_get_backlog(LQL_CLASS(self)));
}

static gboolean
lql_class_helper_fill_stats(LQLClass *self,
			    const struct nlmsghdr *hdr)
{
	const struct tc_stats *stats = NULL;

	g_assert(self != NULL);
	g_assert(hdr != NULL);

	stats = nl_tcmsg_get_rta_val(hdr, TCA_STATS);
	if (stats == NULL) {
		return(FALSE);
	}

	g_print("could update\n");
	

	/* Filter for the class we are looking for since there doesn't appear
	   to be a way to tell the kernel to only return a single class. */
	g_print("handle:%u\n", nl_tcmsg_get_handle(hdr));
	g_print("handle:%u\n", lql_element_get_handle(LQL_ELEMENT(self)));	
 	if ((nl_tcmsg_get_handle(hdr) == lql_element_get_handle(LQL_ELEMENT(self))) &&
	     (nl_tcmsg_get_ifindex(hdr) == lql_element_get_ifIndex(LQL_ELEMENT(self)))) {
		g_print("updating...\n");
		
		self->private->bytes = stats->bytes;
		self->private->packets = stats->packets;
		self->private->drops = stats->drops;
		self->private->overlimits = stats->overlimits;
		self->private->bps = stats->bps;
		self->private->pps = stats->pps;
		self->private->qlen = stats->qlen;
		self->private->backlog = stats->backlog;
	}
	
	return(TRUE);
}

gboolean
lql_class_helper_updateStats(LQLClass *self,
			     LQLCon *con)
{
	TcPacket *pkt = NULL;
	struct nlmsghdr *hdr = NULL;
	const struct nlmsghdr *hdrConst = NULL;
	char *buf = NULL;
	int i = 0, ret = 0;

	g_assert(self != NULL);
	g_assert(con != NULL);
	g_assert(LQL_IS_CLASS(self));

        if (self->private->dispose_has_run) {
                return(FALSE);
        }

	pkt = nl_tcpacket_new();
	hdr = nl_tcpacket_get_nlmsg(pkt);
	
	nl_nlmsg_set_type(hdr, RTM_GETTCLASS);
	nl_nlmsg_set_flags(hdr, NLM_F_REQUEST | NLM_F_ACK | NLM_F_MATCH);
	nl_nlmsg_set_pid(hdr, getpid());
	nl_nlmsg_set_seq(hdr, lql_con_get_inc_sequence(con));

	nl_tcmsg_set_ifindex(hdr, lql_element_get_ifIndex(LQL_ELEMENT(self)));

	if (!nl_send(lql_con_get_nlcon(LQL_CON(con)), pkt, nl_nlmsg_get_length(hdr))) {
		return(FALSE);
	}

	buf = g_malloc0(BUFF);

	for (;;) {
		ret = nl_read(lql_con_get_nlcon(LQL_CON(con)), buf, BUFF);
		i = 0;
		
		while ((hdrConst = nl_get_nlmsghdr(buf, ret, i)) != NULL) {
			if (hdrConst->nlmsg_type == RTM_NEWTCLASS) {
				lql_class_helper_fill_stats(self, hdrConst);
			} else if (hdrConst->nlmsg_type == NLMSG_DONE) {
				// Just so DONE messages don't cause the below print.
			} else {
				g_printerr("Unexpected type:%i\n", hdrConst->nlmsg_type);
			}

			if (nl_nlmsg_is_done(hdrConst)) {
				goto done;
			}

			i++;
		}
	}

 done:
	g_free(buf);
	g_free(pkt);
	
	return(TRUE);
}
